/***************************************************************************/
/*                                                                         */
/* SPX_C16.H : SPX 16 bit Interface -- C library Header file               */
/*              Currently implemented for SPX-EXM-INTF, SPX-PC16-INTF and  */
/*              SPX-PC104-INTF interfaces                                  */
/*                                                                         */
/* Copyright (c) 1993 by Automated Process Control                         */
/*                       106 Business Park Drive                           */
/*                       Jackson, Ms. 39213                                */
/*                       Ph  (601) 956-2800                                */
/*                       FAX (601) 956-9777                                */
/*                                                                         */
/*  Version : 1.00                                                         */
/*  Author  : Philip E. Foster                                             */
/*  Date    : July 16, 1993                                                */
/*                                                                         */
/*  Code Compliance : Microsoft C 6.0a with MS Extensions                  */
/*                    switch selected and level 4 warnings                 */
/*                                                                         */
/***************************************************************************/

/***************************************************************************/
/*                                                                         */
/* Section 1      : Dualport RAM Definitions                               */
/*                                                                         */
/* Specifications : The 16 bit Interface dualport RAM is mapped as a 2K    */
/*                  byte (1Kx16) block of memory between 0x8000:0x0000 and */
/*                  0xFF80:0x0000. The base address for the SPX-EXM-INTF   */
/*                  card is set via the EXM Configuration Menu as well as  */
/*                  default IRQ selection. The SPX-PC16-INTF and           */
/*                  SPX-PC104-INTF have switches to set the base address   */
/*                  and default to <no IRQ> selected                       */
/*                                                                         */
/*                                                                         */
/*                                                                         */
/* Purpose        : The following definitions define the offsets to the    */
/*                  various INPUT, OUTPUT and control TABLES or BUFFERS    */
/*                  as well as the offsets for the COMMAND and STATUS      */
/*                  words used to control the interface. Also included     */
/*                  are the with bit pattern definitions for these words.  */
/*                                                                         */
/*                  NOTE: All offsets are represented as WORD offsets and  */
/*                        the "library" code provided is based upon access */
/*                        to the SPX 16 Bit Interface dualport RAM as an   */
/*                        array of 16 bit wide "short integers".           */
/*                                                                         */
/*                  NOTE: All CONTROL bit patterns are may be ORed to      */
/*                        create the desired value and the STATUS bit      */
/*                        patterns may be used as masks to isolate the     */
/*                        various status bits. These bit patterns are      */
/*                        also based upon an array of 16 bit "short        */
/*                        integers"                                        */
/*                                                                         */
/*                  NOTE: All UNDEFINED locations and bit values are       */
/*                          reserved.                                      */
/*                        All undefined bits in control words MUST be set  */
/*                          to zero.                                       */
/*                        All undefined bits in status words assume any    */
/*                          value and are NOT guaranteed to always be      */
/*                          zero.                                          */
/*                                                                         */
/*                                                                         */
/* Note: The routines and macros defined in this library expect to have a  */
/*       far pointer of SPX_TYPE pointing to the base address for the 2k   */
/*       block of memory on the interface card:                            */
/*         SPX_TYPE far *ifbase;                                           */
/*                                                                         */
/***************************************************************************/

/*******************************************************************************/
/*                                                                             */
/* NOTE: If SPX_TYPE is changed to "char" the bit masks associated with the    */
/*       the ms-byte of a word will have to be modified to take the byte size  */
/*       orientation into account. Offset indexes are automatically corrected. */
/*                                                                             */
/*******************************************************************************/

#define SPX_TYPE            unsigned short   /* for word access              */
#define SPX_SIZE            sizeof(SPX_TYPE) /* this is assumed to be 2      */

#define NON_MUX_IN         (0x0000/SPX_SIZE) /* offset to non-multiplex input buffer   */
#define NON_MUX_OUT        (0x0020/SPX_SIZE) /* offset to non-multiplex output buffer  */
#define NON_MUX_MASK       (0x0040/SPX_SIZE) /* offset to non-multiplex mask buffer    */
#define SAFE_BUFFER        (0x0060/SPX_SIZE) /* offset to safe state buffer            */
#define MUX_IN_MASK_BUF    (0x0080/SPX_SIZE) /* offset to multiplex input mask buffer  */
#define MUX_OUT_MASK_BUF   (0x00A0/SPX_SIZE) /* offset to multiplex output mask buffer */
#define MUX_IN_MASK_WD     (0x00C0/SPX_SIZE) /* offset to multiplex input mask word    */
#define MUX_OUT_MASK_WD    (0x00C2/SPX_SIZE) /* offset to multiplex output mask word   */


#define CONTROL            (0x00F0/SPX_SIZE) /* offset to CONTROL byte                */
#define __MUX_SIZE_ENA      0x4000           /* Multiplex Size mapping ENABLE         */
#define __MUX_SIZE_16       0x3000           /* Select 16 multiplex channels          */
#define __MUX_SIZE_8        0x2000           /* Select  8 multiplex channels          */
#define __MUX_SIZE_4        0x1000           /* Select  4 multiplex channels          */
#define __MUX_SIZE_2        0x0000           /* Select  2 multiplex channels          */
#define __MAP               0x0080           /* Multiplex/Non-multiplex MAP function  */
#define __MAP_PARAMS        0x0020           /* Map Seriplex Parameters               */
#define __MAP_MODE          0x0010           /* Mode Mapping Enable (ORed with        */
                                             /*     __MAP_PARAMS                      */
#define __HOST_WATCH        0x0008           /* Map Host Watchdog or                  */
                                             /*  Host Watch Fault Restart             */
#define __FORCE_LONG        0x0003           /* bit pattern for forced long reset     */
#define __SAFE              0x0002           /* bit pattern for safe state mode       */
#define __MONITOR           0x0001           /* bit pattern for monitor mode          */
#define __NORMAL            0x0000           /* bit pattern for normal operation      */


#define STATUS             (0x00F2/SPX_SIZE) /* offset to STATUS byte                 */
#define STATUS_LS          (0x00F2)          /* offset to STATUS byte                 */
#define __LONG_RST          0x0020           /* LONG_RESET DETECT flag                */
#define __BUSY              0x0010           /* Interface busy flag                   */
#define __MODE_2            0x0001           /* Seriplex Default MODE                 */

#define X_CONTROL          (0x00F4/SPX_SIZE) /* offset to expanded control ls-byte    */
#define __PRIORITY_ENA      0x1000           /* Priority channel enable               */
#define __PRIORITY_CH15     0x0F00           /* Priority channel number 15            */
#define __PRIORITY_CH14     0x0E00           /* Priority channel number 14            */
#define __PRIORITY_CH13     0x0D00           /* Priority channel number 13            */
#define __PRIORITY_CH12     0x0C00           /* Priority channel number 12            */
#define __PRIORITY_CH11     0x0B00           /* Priority channel number 11            */
#define __PRIORITY_CH10     0x0A00           /* Priority channel number 10            */
#define __PRIORITY_CH09     0x0900           /* Priority channel number  9            */
#define __PRIORITY_CH08     0x0800           /* Priority channel number  8            */
#define __PRIORITY_CH07     0x0700           /* Priority channel number  7            */
#define __PRIORITY_CH06     0x0600           /* Priority channel number  6            */
#define __PRIORITY_CH05     0x0500           /* Priority channel number  5            */
#define __PRIORITY_CH04     0x0400           /* Priority channel number  4            */
#define __PRIORITY_CH03     0x0300           /* Priority channel number  3            */
#define __PRIORITY_CH02     0x0200           /* Priority channel number  2            */
#define __PRIORITY_CH01     0x0100           /* Priority channel number  1            */
#define __PRIORITY_CH00     0x0000           /* Priority channel number  0            */
#define __WATCHENA          0x0080           /* HOST Watch Dog Enable                 */
#define __TC_500            0x0030           /* Watchdog Time Constant -- 500mS       */
#define __TC_380            0x0020           /* Watchdog Time Constant -- 380mS       */
#define __TC_240            0x0010           /* Watchdog Time Constant -- 240mS       */
#define __TC_120            0x0000           /* Watchdog Time Constant -- 120mS       */
#define __WATCH_ACK         0x0004           /* HOST Watchdog Fault Acknowledge Bit   */
#define __WATCH_RCVRY       0x0002           /* HOST Watchdog Fault Recovery Method   */
#define __WATCH_ACTION      0x0001           /* HOST Watchdog Fault Action            */

#define X_STATUS           (0x00F6/SPX_SIZE) /* offset to expanded status ls-byte     */
#define __MUX_SIZE_16       0x3000           /* Select 16 multiplex channels          */
#define __MUX_SIZE_8        0x2000           /* Select  8 multiplex channels          */
#define __MUX_SIZE_4        0x1000           /* Select  4 multiplex channels          */
#define __MUX_SIZE_2        0x0000           /* Select  2 multiplex channels          */
#define __MUX_MODE          0x0100           /* interface is in MUX mode              */
#define __HOST_FAULT        0x0080           /* HOST Watchdog Fault has occured       */
#define __HI_CUR_FAULT      0x0020           /* Seriplex HI current fault > 32mA      */
#define __LO_CUR_FAULT      0x0010           /* Seriplex LO current fault < 28mA      */
#define __WATCH_FAULT       0x0008           /* interface WATCHFAULT has occured.     */
#define __BUS_FAULT_OFF     0x0004           /* fault on the Seriplex DATA bus always */
                                             /*   in OFF (12V) state.                 */
#define __BUS_FAULT_ON      0x0002           /* fault on the Seriplex DATA bus always */
                                             /*   in ON (0V) state.                   */
#define __LONG_RESET        0x0001           /* Seriplex Clock has been OFF long      */
                                             /* enough for the modules to detect a    */
                                             /* LONG RESET Condition (approx 40mS)    */
#define __FAULT_MASK       (__HOST_FAULT|__HI_CUR_FAULT|__LO_CUR_FAULT|__WATCH_FAULT|__BUS_FAULT_OFF|__BUS_FAULT_ON|__LONG_RESET)



#define HSPX_PARAMS        (0x00FC/SPX_SIZE) /* offset to the Seriplex Parameters     */
#define __ENA_SINDEL        0x8000           /* enable soft input delay fault         */
#define __HSPX_MODE         0x0080           /* Word Seriplex Bus MODE if __MAP_MODE  */
                                             /*   in CONTROL_LS is set when           */
                                             /*   __MAP_PARAMS in CONTROL_LS is       */
                                             /*   issued .                            */
#define __HSPX_100K         0x0070           /* Seriplex Clock Rate == 100KHz         */
#define __HSPX__75K         0x0060           /* Seriplex Clock Rate ==  75KHz         */
#define __HSPX__64K         0x0050           /* Seriplex Clock Rate ==  64KHz         */
#define __HSPX__50K         0x0040           /* Seriplex Clock Rate ==  50KHz         */
#define __HSPX__32K         0x0030           /* Seriplex Clock Rate ==  32KHz         */
#define __HSPX__25K         0x0020           /* Seriplex Clock Rate ==  25KHz         */
#define __HSPX__20K         0x0010           /* Seriplex Clock Rate ==  20KHz         */
#define __HSPX__16K         0x0000           /* Seriplex Clock Rate ==  16KHz         */

#define __HSPX_256          0x000F           /* Seriplex Frame Size == 256 channels   */
#define __HSPX_240          0x000E           /* Seriplex Frame Size == 240 channels   */
#define __HSPX_224          0x000D           /* Seriplex Frame Size == 224 channels   */
#define __HSPX_208          0x000C           /* Seriplex Frame Size == 208 channels   */
#define __HSPX_192          0x000B           /* Seriplex Frame Size == 192 channels   */
#define __HSPX_176          0x000A           /* Seriplex Frame Size == 176 channels   */
#define __HSPX_160          0x0009           /* Seriplex Frame Size == 160 channels   */
#define __HSPX_144          0x0008           /* Seriplex Frame Size == 144 channels   */
#define __HSPX_128          0x0007           /* Seriplex Frame Size == 128 channels   */
#define __HSPX_112          0x0006           /* Seriplex Frame Size == 112 channels   */
#define __HSPX__96          0x0005           /* Seriplex Frame Size ==  96 channels   */
#define __HSPX__80          0x0004           /* Seriplex Frame Size ==  80 channels   */
#define __HSPX__64          0x0003           /* Seriplex Frame Size ==  64 channels   */
#define __HSPX__48          0x0002           /* Seriplex Frame Size ==  48 channels   */
#define __HSPX__32          0x0001           /* Seriplex Frame Size ==  32 channels   */
#define __HSPX__16          0x0000           /* Seriplex Frame Size ==  16 channels   */


#define ID_WORD            (0x00FE/SPX_SIZE) /* offset to the Interface Software ID   */
                                             /*   NOTE: The digits of ID_WORD are in  */
                                             /*         PACKED BDC format.            */
                                             /*         Ie. Version 1.10 is in memory */
                                             /*             as 0x10, 0x01             */

/*                                                                                    */
/* NOTE: mux buffers begin with MODULE 1  (network addr. 16)  - CHANNEL 0             */
/*               and go through MODULE 15 (network addr. 240) - CHANNEL 15            */
/*       ie. 240 analog channels requiring 480 (1E0h) bytes                           */
/*                                                                                    */

#define MUX_IN             (0x0100/SPX_SIZE) /* offset to analog input buffer         */
#define MUX_OUT            (0x0300/SPX_SIZE) /* offset to analog output buffer        */
#define MUX_MASK           (0x0500/SPX_SIZE) /* offset to analog mask buffer          */

#define IRQ_SEL            (0x07F4/SPX_SIZE) /* IRQ select                            */
#define __NoIRQ             0x0000           /* No IRQ selected                       */
#define __Irq3              0x0100           /* IRQ3                                  */
#define __Irq4              0x0200           /* IRQ3                                  */
#define __Irq5              0x0300           /* IRQ3                                  */
#define __Irq6              0x0400           /* IRQ3                                  */
#define __Irq7              0x0500           /* IRQ3                                  */
#define __Irq14             0x0600           /* IRQ3                                  */
#define __Irq15             0x0700           /* IRQ3                                  */
#define __AckRD             0x0000           /* Acknowledge IRQ on Read INT_STATUS    */
#define __AckWR             0x8000           /* Acknowledge IRQ on Write INT_STATUS   */

#define HOST_RESET         (0x07F6/SPX_SIZE) /* HOST controlled local reset to        */
#define __HOST_RESET        0x5500           /*  interface when __HOST_RESET is sent  */

#define HOST_WATCHDOG      (0x07F8/SPX_SIZE) /* HOST Watchdog Event is issued when    */
#define __HOST_WATCHDOG     0x5500           /*  __HOST_WATCHDOG is sent              */

#define INT_STATUS         (0x07FC/SPX_SIZE) /* offset to interrupt status byte       */
#define INT_ENABLE         (0x07FE/SPX_SIZE) /* offset to interrupt enable byte       */
#define __CMD_INT           0x0004           /* bit mask/pattern for command int      */
#define __FAULT_INT         0x0002           /* bit mask/pattern for fault int        */
#define __FRAME_INT         0x0001           /* bit mask/pattern for frame int        */


/***************************************************************************/
/*                                                                         */
/* Section 2 : C Library Routine macros, definitions and function          */
/*             prototypes.                                                 */
/*                                                                         */
/***************************************************************************/


/* NOTE: The timeout times represented by NBUSY_TIME and BUSY_TIME below   */
/*       will need to be varied according to the speed of the processor    */
/*       in use. The currently active times work on an Intel 485DX2 at     */
/*       50 MHz. The values in coments work on an Intel 386DX at 16 MHz.   */
/*       Be sure and adjust these values according to the machine they     */
/*       will be used on or modify the code to handle the timeout          */
/*       functions according to your system requirements.                  */

#ifndef NBUSY_TIME
#define NBUSY_TIME  (unsigned long)150000       /* 50000  */
#endif
#ifndef BUSY_TIME
#define BUSY_TIME   (unsigned long)50000        /* 20000  */
#endif

#ifndef ON
#define ON  1
#endif
#ifndef OFF
#define OFF 0
#endif

/* NOTE: The values for SUCCESS, TIMEOUT_NB and TIMEOUT_B are returned   */
/*       by the routines with timeout functions in them.                 */

#define SUCCESS     0x00        /* no problem                           */
#define TIMEOUT_NB  0xE0        /* time out while waiting for not busy  */
#define TIMEOUT_B   0xE1        /* time out while waiting for busy      */


/********************************************************/
/*                                                      */
/*  Bit Mask data array for bit manipulation            */
/*                                                      */
/* bitval[16]                                           */
/*                                                      */
/*                                                      */
/*  Source  : SPX_DATA.C                                */
/*                                                      */
/********************************************************/
extern unsigned short bitval[];


/********************************************************/
/*                                                      */
/*  Non-Multiplexed Input Routine                       */
/*                                                      */
/* nmux_in(ifbase, address)                             */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*                                                      */
/*  Returns    : 0 if input is OFF                      */
/*               1 if input is ON                       */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux_in(ifbase, address)                ((ifbase[NON_MUX_IN+(address>>4)] & bitval[(address&0x0F)]) ? 1 : 0)


/********************************************************/
/*                                                      */
/*  Non-Multiplexed Output Routine to turn channel OFF  */
/*                                                      */
/* nmux_off(ifbase, address)                            */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux_off(ifbase, address)               (ifbase[NON_MUX_OUT+(address>>4)] &= ~bitval[(address&0x0F)])

/********************************************************/
/*                                                      */
/*  Non-Multiplexed Output Routine to turn channel ON   */
/*                                                      */
/* nmux_on(ifbase, address)                             */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux_on(ifbase, address)                (ifbase[NON_MUX_OUT+(address>>4)] |= bitval[(address&0x0F)])

/********************************************************/
/*                                                      */
/*  Non-Multiplexed Output Routine (1 bit)              */
/*                                                      */
/* nmux_out(ifbase, address, action)                    */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*               action defines the action to take on   */
/*                      the output. 0 == OFF,           */
/*                      NON-0 == ON                     */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux_out(ifbase, address, action)       (action ? nmux_on(ifbase, address) : nmux_off(ifbase, address))

/********************************************************/
/*                                                      */
/*  Non-Multiplexed Input Routine (8 bit)               */
/*                                                      */
/* nmux8_in(ifbase, address)                            */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (8-248) representing    */
/*                      the base (beginning) address    */
/*                      to read from                    */
/*                                                      */
/*  Returns    : 8 bit value associated with selected   */
/*                      network addresses               */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux8_in(ifbase, address)               ((address&0x0010)?(ifbase[NON_MUX_IN+(address>>4)]>>8)&0xff:(ifbase[NON_MUX_IN+(address>>4)]&0xff))

/********************************************************/
/*                                                      */
/*  Non-Multiplexed Read Output Buffer Routine (8 bit)  */
/*                                                      */
/* nmux8_getout(ifbase, address)                        */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (8-248) representing    */
/*                      the base (beginning) address    */
/*                      to read from                    */
/*                                                      */
/*  Returns    : 8 bit value associated with selected   */
/*                      network addresses               */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux8_getout(ifbase, address)           ((address&0x0010)?(ifbase[NON_MUX_OUT+(address>>4)]>>8)&0xff:(ifbase[NON_MUX_IN+(address>>4)]&0xff))

/********************************************************/
/*                                                      */
/*  Non-Multiplexed Read Passthru Buffer (1 bit)        */
/*                                                      */
/* nmux_mask_in(ifbase, address)                        */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*                                                      */
/*  Returns    : 0 if input is OFF                      */
/*               1 if input is ON                       */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux_mask_in(ifbase, address)           ((ifbase[NON_MUX_MASK+(address>>4)] & bitval[(address&0x0F)]) ? 1 : 0)

/********************************************************/
/*                                                      */
/*  Non-Multiplexed Write Passthru Buffer (1 bit)       */
/*                                                      */
/* nmux_mask_out(ifbase, address, action)               */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*               action defines the action to take on   */
/*                      the output. 0 == OFF,           */
/*                      NON-0 == ON                     */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define nmux_mask_out(ifbase, address, action)  (action ? (ifbase[NON_MUX_MASK+(address>>4)] |= bitval[(address&0x0F)]) : (ifbase[NON_MUX_MASK+(address>>4)] &= ~bitval[(address&0x0F)]))

/********************************************************/
/*                                                      */
/*  Read Safe State Buffer (1 bit)                      */
/*                                                      */
/* safe_in(ifbase, address)                             */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*                                                      */
/*  Returns    : 0 if input is OFF                      */
/*               1 if input is ON                       */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define safe_in(ifbase, address)                ((ifbase[SAFE_BUFFER+(address>>4)] & bitval[(address&0x0F)]) ? 1 : 0)

/********************************************************/
/*                                                      */
/*  Write Safe State Buffer (1 bit)                     */
/*                                                      */
/* afe_out(ifbase, address, action)                     */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (0-255)                 */
/*               action defines the action to take on   */
/*                      the output. 0 == OFF,           */
/*                      NON-0 == ON                     */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define safe_out(ifbase, address, action)       (action ? (ifbase[SAFE_BUFFER+(address>>4)] |= bitval[(address&0x0F)]) : (ifbase[SAFE_BUFFER+(address>>4)] &= ~bitval[(address&0x0F)]))

/********************************************************/
/*                                                      */
/*  Read Multiplex Channel Input (16 bits)              */
/*                                                      */
/* mux_in(ifbase, module, channel)                      */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               module is the base Seriplex network    */
/*                      address (16-240) representing   */
/*                      the base (beginning) address    */
/*                      of the module                   */
/*               channel is the module channel address  */
/*                      (0-15)                          */
/*                                                      */
/*  Returns    : 16 bit value associated with the       */
/*                      selected module and channel.    */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define mux_in(ifbase, module, channel)         (ifbase[MUX_IN+(module-16)+channel])

/********************************************************/
/*                                                      */
/*  Write Multiplex Channel Output (16 bits)            */
/*                                                      */
/* mux_out(ifbase, module, channel, value)              */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               module is the base Seriplex network    */
/*                      address (16-240) representing   */
/*                      the base (beginning) address    */
/*                      of the module                   */
/*               channel is the module channel address  */
/*                      (0-15)                          */
/*               value is the value to output to the    */
/*                      selected module and channel.    */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define mux_out(ifbase, module, channel, value) (ifbase[MUX_OUT+(module-16)+channel] = value)

/********************************************************/
/*                                                      */
/*  Read Multiplex Channel Output Buffer (16 bits)      */
/*                                                      */
/* get_muxout(ifbase, module, channel)                  */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               module is the base Seriplex network    */
/*                      address (16-240) representing   */
/*                      the base (beginning) address    */
/*                      of the module                   */
/*               channel is the module channel address  */
/*                      (0-15)                          */
/*                                                      */
/*  Returns    : 16 bit value associated with the       */
/*                      selected module and channel.    */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define get_muxout(ifbase, module, channel)     (ifbase[MUX_OUT+(module-16)+channel])

/********************************************************/
/*                                                      */
/*  Write Multiplex Channel Passthru Buffer (16 bits)   */
/*                                                      */
/*  mux_mask_out(ifbase, module, channel, action)       */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               module is the base Seriplex network    */
/*                      address (16-240) representing   */
/*                      the base (beginning) address    */
/*                      of the module                   */
/*               channel is the module channel address  */
/*                      (0-15)                          */
/*               action defines whether to set the      */
/*                      selected module and channel     */
/*                      to pass data thru if NON-0,     */
/*                      else sets as not passed thru.   */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define mux_mask_out(ifbase, module, channel, action) (ifbase[MUX_MASK+(module-16))+channel] = (action?0xFFFF:0))

/********************************************************/
/*                                                      */
/*  Write Interface CONTROL word                        */
/*                                                      */
/*  if_cntrl(ifbase, value)                             */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               value is the value for CONTROL.        */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_cntrl(ifbase, value)                 (ifbase[CONTROL] = value)

/********************************************************/
/*                                                      */
/*  Read Interface CONTROL word                         */
/*                                                      */
/*  ifif_get_cntrl(ifbase)                              */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of CONTROL.                      */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_get_cntrl(ifbase)                    (ifbase[CONTROL])

/********************************************************/
/*                                                      */
/*  Write Interface X_CONTROL word                      */
/*                                                      */
/*  if_cntrl(ifbase, value)                             */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               value is the value for X_CONTROL.      */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_xcntrl(ifbase, value)                (ifbase[X_CONTROL] = value)

/********************************************************/
/*                                                      */
/*  Read Interface X_CONTROL word                       */
/*                                                      */
/*  if_get_cntrl(ifbase)                                */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of X_CONTROL.                    */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_get_xcntrl(ifbase)                   (ifbase[X_CONTROL])

/********************************************************/
/*                                                      */
/*  Read Interface STATUS word                          */
/*                                                      */
/*  if_stat(ifbase)                                     */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of STATUS.                       */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_stat(ifbase)                         (ifbase[STATUS])

/********************************************************/
/*                                                      */
/*  Read Interface STATUS word with a bit mask          */
/*                                                      */
/*  if_statb(ifbase, mask)                              */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               mask is the bit mask for the desired   */
/*                      STATUS bits.                    */
/*                                                      */
/*  Returns    : value of STATUS & mask.                */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_statb(ifbase, mask)                  (ifbase[STATUS] & mask)

/********************************************************/
/*                                                      */
/*  Read Interface X_STATUS word                        */
/*                                                      */
/*  if_stat(ifbase)                                     */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of X_STATUS.                     */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_xstatl(ifbase)                       (ifbase[X_STATUS])

/********************************************************/
/*                                                      */
/*  Read Interface X_STATUS word with a bit mask        */
/*                                                      */
/*  if_statb(ifbase, mask)                              */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               mask is the bit mask for the desired   */
/*                      X_STATUS bits.                  */
/*                                                      */
/*  Returns    : value of X_STATUS & mask.              */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_xstatlb(ifbase, mask)                (ifbase[X_STATUS] & mask)

/********************************************************/
/*                                                      */
/*  Write Multiplexed Output Mask Word                  */
/*     (MUX_OUT_MASK_WD)                                */
/*                                                      */
/*  muxout_wmask(ifbase, mask)                          */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               mask is a bit map indicating which     */
/*                      module(s) (16 bit groups) are   */
/*                      to be treated as Multiplexed    */
/*                      outputs.                        */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define muxout_wmask(ifbase, mask)              (ifbase[MUX_OUT_MASK_WD] = mask)

/********************************************************/
/*                                                      */
/*  Write Multiplexed Input Mask Word                   */
/*     (MUX_IN_MASK_WD)                                 */
/*                                                      */
/*  muxin_wmask(ifbase, mask)                           */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               mask is a bit map indicating which     */
/*                      module(s) (16 bit groups) are   */
/*                      to be treated as Multiplexed    */
/*                      inputs.                         */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define muxin_wmask(ifbase, mask)               (ifbase[MUX_IN_MASK_WD] = mask)

/********************************************************/
/*                                                      */
/*  Write HOST Seriplex Parameters Word                 */
/*      HSPX_PARAMS                                     */
/*                                                      */
/*  set_param(ifbase, value)                            */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               value is the value for HSPX_PARAMS.    */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define set_param(ifbase, value)                (ifbase[HSPX_PARAMS] = value)

/********************************************************/
/*                                                      */
/*  Read HOST Seriplex Parameters Word                  */
/*      HSPX_PARAMS                                     */
/*                                                      */
/*  get_param(ifbase)                                   */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of HSPX_PARAMS.                  */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define get_param(ifbase)                       (ifbase[HSPX_PARAMS])

/********************************************************/
/*                                                      */
/*  Read Interface Software Revision ID Word            */
/*                                                      */
/*  get_ID(ifbase)                                      */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of ID_WORD.                      */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define get_ID(ifbase)                          (ifbase[ID_WORD])

/********************************************************/
/*                                                      */
/*  Write "Interrupt" Control Word (Currently only      */
/*      usable for polling) INT_ENABLE                  */
/*                                                      */
/*  sint_ctrl(ifbase, value)                            */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               value is the value for INT_ENABLE.     */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define int_ctrl(ifbase, value)                 (ifbase[INT_ENABLE] = value)

/********************************************************/
/*                                                      */
/*  Read "Interrupt" Control Word (Currently only       */
/*      usable for polling) INT_ENABLE                  */
/*                                                      */
/*  int_rctrl(ifbase)                                   */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : value of INT_ENABLE.                   */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define int_rctrl(ifbase)                       (ifbase[INT_ENABLE])

/********************************************************/
/*                                                      */
/*  Read "Interrupt" Status Word with bit mask          */
/*      (Currently only usable for polling) INT_STATUS  */
/*                                                      */
/*  iint_stat(ifbase, mask)                             */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               mask is the bit mask for the desired   */
/*                      INT_STATUS bits.                */
/*                                                      */
/*  Returns    : value of INT_STATUS & mask.            */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define int_stat(ifbase, mask)                  (ifbase[INT_STATUS] & mask)

/********************************************************/
/*                                                      */
/*  Reset "Interrupt" Status Word Bit(s) (acknowledges  */
/*      "interrupt" event) INT_ENABLE                   */
/*                                                      */
/*  srst_int_stat(ifbase, mask)                         */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*               mask is the bit mask for the desired   */
/*                      bit(s) to be reset in the       */
/*                      INT_STATUS word.                */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define rst_int_stat(ifbase, mask)              (ifbase[INT_STATUS] &= ~mask)

/********************************************************/
/*                                                      */
/*  Enable End Of Frame "Interrupt" (for Polling)       */
/*                                                      */
/* ena_eof_poll(ifbase)                                 */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define ena_eof_poll(ifbase)                    (ifbase[INT_ENABLE] |= __FRAME_INT)

/********************************************************/
/*                                                      */
/*  Issue a HOST driven "RESET" to the SPX 16 Bit       */
/*   Interface                                          */
/*     used by reset() routine below.                   */
/*                                                      */
/* if_reset(ifbase)                                     */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define if_reset(ifbase)                        (ifbase[HOST_RESET] = __HOST_RESET)

/********************************************************/
/*                                                      */
/*  Issue a HOST "Watchdog Event" to the SPX 16 Bit     */
/*   Interface                                          */
/*                                                      */
/* watch_event(ifbase)                                  */
/*                                                      */
/*  Parameters : ifbase is a "short unsigned *" to the  */
/*                      base address of the interface   */
/*                      array.                          */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_EXM.H -- #define'd                    */
/*                                                      */
/********************************************************/
#define watch_event(ifbase)                     (ifbase[HOST_WATCHDOG] = __HOST_WATCHDOG)

/********************************************************/
/*                                                      */
/*  Wait for End Of Frame (Polling Routine)             */
/*                                                      */
/* wait_eof(ifbase, count)                              */
/*                                                      */
/*  Parameters : ifbase is a "SPX_TYPE *" to the base   */
/*                      address of the interface array. */
/*               count is the number of EOFs to wait    */
/*                      before returning to the caller  */
/*                                                      */
/*  Returns    : 0 if successful                        */
/*               X_STATUS masked to the fault status    */
/*                      bits if unsuccessful            */
/*                                                      */
/*  Source  : SPX_EOFP.C                                */
/*                                                      */
/********************************************************/
unsigned short wait_eof(SPX_TYPE far *ifbase, int count);

/*********************************************************/
/*                                                       */
/*  Routine to Clear (set to 0) all SPX 16 Bit Interface */
/*   Buffers                                             */
/*                                                       */
/* if_clear(ifbase)                                      */
/*                                                       */
/*  Parameters : ifbase is a "SPX_TYPE far*" to the base */
/*                      address of the interface array.  */
/*                                                       */
/*  Returns    : <none>                                  */
/*                                                       */
/*  Source  : SPX_CLR.C                                  */
/*                                                       */
/*********************************************************/
void if_clear(SPX_TYPE far *ifbase);

/********************************************************/
/*                                                      */
/*  Routine to output 8 bit NON-MUX values              */
/*                                                      */
/* nmux8_out(ifbase, address, value)                    */
/*                                                      */
/*  Parameters : ifbase is a "SPX_TYPE far *" to the    */
/*                      base address of the interface   */
/*                      array.                          */
/*               address is the Seriplex network        */
/*                      address (8-248) representing    */
/*                      the base (beginning) address    */
/*                      to read from                    */
/*               value is the 8 bit value to output.    */
/*                                                      */
/*  Returns    : <none>                                 */
/*                                                      */
/*  Source  : SPX_NM8O.C                                */
/*                                                      */
/********************************************************/
void nmux8_out(SPX_TYPE far *ifbase, unsigned short address,
               unsigned char value);

/********************************************************/
/*                                                      */
/*  Interface Control and Mapping Routine               */
/*                                                      */
/* if_map(ifbase, control, busy_tim, nbusy_tim)         */
/*                                                      */
/*  Parameters : ifbase is a "SPX_TYPE fsr *" to the    */
/*                      base address of the interface   */
/*                      array.                          */
/*               control is the value to write to the   */
/*                      CONTROL word.                   */
/*               busy_tim is the "timeout" value used   */
/*                      while waiting for the interface */
/*                      to indicate busy after writing  */
/*                      CONTROL                         */
/*               nbusy_tim is the "timeout" value used  */
/*                      while waiting for the interface */
/*                      to be NOT busy before writing   */
/*                      CONTROL                         */
/*  Returns    : SUCCESS if successful                  */
/*               TIMEOUT_NB if a timeout occured while  */
/*                      waiting for the interface to be */
/*                      NOT busy.                       */
/*               TIMEOUT_B if a timeout occured while   */
/*                      waiting for the interface to    */
/*                      become busy.                    */
/*                                                      */
/*  Source  : SPX_CTRL.C                                */
/*                                                      */
/********************************************************/
int  if_map(SPX_TYPE far *ifbase, unsigned short control,
            unsigned long busy_tim, unsigned long nbusy_tim);

/********************************************************/
/*                                                      */
/*  RESETING the SPX 16 Bit Interface                   */
/*                                                      */
/* reset(ifbase, busy_tim)                              */
/*                                                      */
/*  Parameters : ifbase is a "SPX_TYPE far *" to the    */
/*                      base address of the interface   */
/*                      array.                          */
/*               busy_tim is the "timeout" value used   */
/*                      while waiting for the interface */
/*                      to indicate busy after writing  */
/*                      CONTROL                         */
/*  Returns    : SUCCESS if successful                  */
/*               TIMEOUT_B if a timeout occured while   */
/*                      waiting for the interface to    */
/*                      become busy.                    */
/*                                                      */
/*  Source  : SPX_CLR.C                                 */
/*                                                      */
/********************************************************/
int reset(SPX_TYPE far *ifbase, unsigned long busy_tim);

/********************************************************/
/*                                                      */
/*  Wait for Interface NOT BUSY                         */
/*                                                      */
/* wait_nb(ifbase, nbusy_tim)                           */
/*                                                      */
/*  Parameters : ifbase is a "SPX_TYPE far *" to the    */
/*                      base address of the interface   */
/*                      array.                          */
/*               nbusy_tim is the "timeout" value used  */
/*                      while waiting for the interface */
/*                      to be NOT busy before writing   */
/*                      CONTROL                         */
/*  Returns    : SUCCESS if successful                  */
/*               TIMEOUT_NB if a timeout occured while  */
/*                      waiting for the interface to be */
/*                      NOT busy.                       */
/*                                                      */
/*  Source  : SPX_WNB.C                                 */
/*                                                      */
/********************************************************/
int wait_nb(SPX_TYPE far *ifbase, unsigned long nbusy_tim);


/***************************************************************************/
/*                                                                         */
/* SPX_C16.H : SPX 16 Bit Interface -- C library Header file --------      */
/*                    --------- end_of_file ---------                      */
/*                                                                         */
/***************************************************************************/
